@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public 
+ * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  */
@@ -32,6 +32,7 @@
 #include <unistd.h>
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
+#include <selinux/label.h>
 #endif
 #define LU_DEFAULT_SALT_TYPE "$1$"
 #define LU_DEFAULT_SALT_LEN  8
@@ -43,6 +44,21 @@
 #define HASH_ROUNDS_MIN 1000
 #define HASH_ROUNDS_MAX 999999999
 
+#if (defined CRYPT_GENSALT_IMPLEMENTS_AUTO_ENTROPY && \
+     CRYPT_GENSALT_IMPLEMENTS_AUTO_ENTROPY)
+#define USE_XCRYPT_GENSALT 1
+#else
+#define USE_XCRYPT_GENSALT 0
+#endif
+
+#if ((defined XCRYPT_VERSION_NUM && \
+      XCRYPT_VERSION_NUM >= ((4 << 16) | 3)) && \
+      USE_XCRYPT_GENSALT)
+#define HAVE_YESCRYPT 1
+#else
+#define HAVE_YESCRYPT 0
+#endif
+
 struct lu_lock {
 	int fd;
 	struct flock lock;
@@ -66,6 +82,7 @@ lu_strcmp(gconstpointer v1, gconstpointer v2)
 	return strcmp((char *) v1, (char *) v2);
 }
 
+#if !USE_XCRYPT_GENSALT
 /* A list of allowed salt characters, according to SUSv2. */
 #define ACCEPTABLE "ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
 		   "abcdefghijklmnopqrstuvwxyz" \
@@ -115,6 +132,7 @@ fill_urandom(char *output, size_t length)
 	close(fd);
 	return TRUE;
 }
+#endif
 
 static const struct {
 	const char initial[5];
@@ -124,9 +142,12 @@ static const struct {
 } salt_type_info[] = {
 	{"$1$", "$", 8, FALSE },
 	/* FIXME: number of rounds, base64 of 128 bits */
-	{"$2a$", "$", 8, FALSE },
+	{"$2b$", "$", 8, FALSE },
 	{"$5$", "$", 16, TRUE },
 	{"$6$", "$", 16, TRUE },
+#if HAVE_YESCRYPT
+	{"$y$", "$", 24, FALSE },
+#endif
 	{ "", "", 2 },
 };
 
@@ -135,6 +156,9 @@ lu_make_crypted(const char *plain, const char *previous)
 {
 	char salt[2048];
 	size_t i, len = 0;
+#if USE_XCRYPT_GENSALT
+	unsigned long rounds = 0;
+#endif
 
 	if (previous == NULL) {
 		previous = LU_DEFAULT_SALT_TYPE;
@@ -151,6 +175,23 @@ lu_make_crypted(const char *plain, const char *previous)
 
 	if (salt_type_info[i].sha_rounds != FALSE
 	    && strncmp(previous + len, "rounds=", strlen("rounds=")) == 0) {
+#if USE_XCRYPT_GENSALT
+		const char *start;
+		char *end;
+
+		start = previous + len + strlen("rounds=");
+		rounds = strtoul (start, &end, 10);
+
+		if (rounds < HASH_ROUNDS_MIN)
+			rounds = HASH_ROUNDS_MIN;
+		else if (rounds > HASH_ROUNDS_MAX)
+			rounds = HASH_ROUNDS_MAX;
+	}
+
+	g_assert(CRYPT_GENSALT_OUTPUT_SIZE <= sizeof(salt));
+
+	crypt_gensalt_rn(previous, rounds, NULL, 0, salt, sizeof(salt));
+#else
 		const char *start, *end;
 
 		start = previous + len + strlen("rounds=");
@@ -168,6 +209,7 @@ lu_make_crypted(const char *plain, const char *previous)
 		return NULL;
 	strcpy(salt + len + salt_type_info[i].salt_length,
 	       salt_type_info[i].separator);
+#endif
 
 	return crypt(plain, salt);
 }
@@ -231,9 +273,12 @@ lu_util_default_salt_specifier(struct lu_context *context)
 	} salt_types[] = {
 		{ "des", "", FALSE },
 		{ "md5", "$1$", FALSE },
-		{ "blowfish", "$2a$", FALSE },
+		{ "blowfish", "$2b$", FALSE },
 		{ "sha256", "$5$", TRUE },
 		{ "sha512", "$6$", TRUE },
+#if HAVE_YESCRYPT
+		{ "yescrypt", "$y$", FALSE },
+#endif
 	};
 
 	const char *salt_type;
@@ -251,13 +296,18 @@ lu_util_default_salt_specifier(struct lu_context *context)
 
 found:
 	if (salt_types[i].sha_rounds != FALSE) {
-		unsigned long rounds;
+		unsigned long rounds = 0;
 
 		rounds = select_hash_rounds(context);
+#if USE_XCRYPT_GENSALT
+		return g_strdup(crypt_gensalt(salt_types[i].initializer,
+					      rounds, NULL, 0));
+#else
 		if (rounds != 0)
 			return g_strdup_printf("%srounds=%lu$",
 					       salt_types[i].initializer,
 					       rounds);
+#endif
 	}
 	return g_strdup(salt_types[i].initializer);
 }
@@ -696,7 +746,7 @@ lu_util_update_shadow_last_change(struct lu_ent *ent)
 #ifdef WITH_SELINUX
 /* Store current fscreate context to ctx. */
 gboolean
-lu_util_fscreate_save(security_context_t *ctx, struct lu_error **error)
+lu_util_fscreate_save(char **ctx, struct lu_error **error)
 {
 	*ctx = NULL;
 	if (is_selinux_enabled() > 0 && getfscreatecon(ctx) < 0) {
@@ -710,7 +760,7 @@ lu_util_fscreate_save(security_context_t *ctx, struct lu_error **error)
 
 /* Restore fscreate context from ctx, and free it. */
 void
-lu_util_fscreate_restore(security_context_t ctx)
+lu_util_fscreate_restore(char *ctx)
 {
 	if (is_selinux_enabled() > 0) {
 		(void)setfscreatecon(ctx);
@@ -724,7 +774,7 @@ gboolean
 lu_util_fscreate_from_fd(int fd, const char *path, struct lu_error **error)
 {
 	if (is_selinux_enabled() > 0) {
-		security_context_t ctx;
+		char *ctx;
 
 		if (fgetfilecon(fd, &ctx) < 0) {
 			lu_error_new(error, lu_error_stat,
@@ -750,7 +800,7 @@ gboolean
 lu_util_fscreate_from_file(const char *file, struct lu_error **error)
 {
 	if (is_selinux_enabled() > 0) {
-		security_context_t ctx;
+		char *ctx;
 
 		if (getfilecon(file, &ctx) < 0) {
 			lu_error_new(error, lu_error_stat,
@@ -776,7 +826,7 @@ gboolean
 lu_util_fscreate_from_lfile(const char *file, struct lu_error **error)
 {
 	if (is_selinux_enabled() > 0) {
-		security_context_t ctx;
+		char *ctx;
 
 		if (lgetfilecon(file, &ctx) < 0) {
 			lu_error_new(error, lu_error_stat,
@@ -803,9 +853,18 @@ lu_util_fscreate_for_path(const char *path, mode_t mode,
 			  struct lu_error **error)
 {
 	if (is_selinux_enabled() > 0) {
-		security_context_t ctx;
+        char *ctx;
+        struct selabel_handle *label_handle = NULL;
 
-		if (matchpathcon(path, mode, &ctx) < 0) {
+        label_handle = selabel_open(SELABEL_CTX_FILE, NULL, 0);
+        if (!label_handle) {
+            lu_error_new(error, lu_error_open,
+					     _("couldn't obtain selabel file "
+					       "context handle: %s"),
+					     strerror(errno));
+            return FALSE;
+        }
+        if (selabel_lookup(label_handle, &ctx, path, mode) < 0) {
 			if (errno == ENOENT)
 				ctx = NULL;
 			else {
@@ -813,9 +872,11 @@ lu_util_fscreate_for_path(const char *path, mode_t mode,
 					     _("couldn't determine security "
 					       "context for `%s': %s"), path,
 					     strerror(errno));
+                selabel_close(label_handle);
 				return FALSE;
 			}
 		}
+        selabel_close(label_handle);
 		if (setfscreatecon(ctx) < 0) {
 			lu_error_new(error, lu_error_generic,
 				     _("couldn't set default security context "
